/*
  Name: GPS Compass
  Copyright: 
  Author: Georgi N. Chakarov
  Date: 21.07.08 13:22
  Description: Inputs an angle in degrees and creates a compass
               in an array and displays it. 
*/

//#include <iostream>
#include <math.h>
//using namespace std;
#include "compass/trigonometry.h"
#include "lcd/lcd.h"

#define CENTERX 12  //r+1
#define CENTERY 12
#define r       11
#define PI      3.1415926535897932384626433832795
#define const_center 29 //42-CENTERY


static int n=23;
static char a[5][26];
//volatile int alpha=0;  //the arrow angle relative to North
static void draw_circle();
static void clear();
static void draw_compass(int alpha, int num);


static void write_a(int x, int y)
{
    a[x/8][y] |= (1<<x%8);
}

static double dist(int x1, int y1, float x2, float y2)
{return sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1))*1.0;}

static void draw_circle()
{
   double d1, d2;
   char ch;
   int i, j;
   for(i=CENTERX-r; i<=CENTERX; i++)
	   for(j=CENTERY; j<=CENTERY+r; j++)
  		{
		 	d1=dist(CENTERX, CENTERY, i-0.39, j+0.39);
	        d2=dist(CENTERX, CENTERY, i+0.39, j-0.39); 
			if(d2<r && r<d1)
			{	
 		       write_a(i, j);
               write_a(2*CENTERX-i, j);
 		       write_a(i, 2*CENTERY-j);
			   write_a(j, i);
	        }
	    }  	 
}

static void print(char mode)
{ 
 
 /* 
 //	   LCDInit();
 //	   LCDPixel();
 	  int i, j;
     cout << "  " << endl;
     for (i=0; i<n; i++)
 	 {
	  	 for(j=0; j<n; j++)
	  	 {
  		    if(a[i][j]==0) cout << "  ";
  		    else cout << "* ";
         }
         cout << endl;
     }
     cout << endl << endl;
 */

char pr, k;
if(mode!=2)
{ 
	LCDChrXY(39, 0, 'N');
	LCDChrXY(39, 4, 's');
	LCDChrXY(62, 2, 'E');
	LCDChrXY(17, 2, 'W');
}
if(mode==2) mode=0;
else mode=1;

for(pr=0; pr<=n/8; pr++)
{
  LCDSend(0x40|pr+mode, SEND_CMD );
  for(k=0; k<=n; k++) {
    
    LCDSend(0x80|k+const_center, SEND_CMD );
    LCDSend(a[pr][k], SEND_CHR);    
  }  
 }
} 

static void clear()
{
    int i, j;	
    for(i=0; i<5; i++)
 	   for(j=0; j<26; j++)
 	      a[i][j]=0;
}
static int abs(int x)
{
  if(x>=0) return x;
  else return -x;
}

static void draw_compass(int alpha, int num)
{
   int x1, x2, y1, y2;
   int y1s, x1s, x1t;
   int swap;
   int incx, incy;
   double cotga;   //cotangens alpha
	 x1=CENTERX;
	 y1=CENTERY;
	 if(alpha==0 || alpha==360){x2=CENTERX-r; y2=CENTERY;}
	 if(alpha>0 && alpha<180)
	 {
	   x2=CENTERX-cosa(alpha)*r;
	   y2=CENTERY+sina(alpha)*r;
	 }
     if(alpha==90){int l; for(l=CENTERY; l<=CENTERY+r; l++) write_a(CENTERX, l);}
	 if(alpha==180){x2=CENTERX+r; y2=CENTERY;}
	 if(alpha==270){int h; for(h=CENTERY-r; h<=CENTERY; h++) write_a(CENTERX, h);}
	 if(alpha>180 && alpha<360)
	 {
	   
	   x2=CENTERX+cosa((alpha-180))*r;
	   y2=CENTERY-sina((alpha-180))*r;
	 }
	 
	 x1s=x1; y1s=y1;
	 
	
	 if(y1>y2)
	 {
        swap=x1; x1=x2; x2=swap;
        swap=y1; y1=y2; y2=swap;
     }
	 
	 
      incx=abs(x2-x1);
      incy=abs(y2-y1);
	 cotga=(double)incx/incy;
	
	 if(x1 < x2)
	 while(x1 < x2+1 && y1< y2+1)
	 {
        if((double)(x1-x1s+1)==(double)(y1-y1s+1)*cotga)
	  	{
			write_a(x1, y1);
			x1++; y1++;
			continue;
		}
	    if((double)(x1-x1s+1)/cotga > (y1-y1s+1))
	  	{
	      write_a(x1, y1);
	      y1++;
          continue;
		}
		if((double)(y1-y1s+1)*cotga > (x1-x1s+1))
		{
           write_a(x1, y1);
           x1++;
           continue;
        }
     }
     
     if(x1>x2)
     {
	  x1t=x1;
	  swap=x1; x1=x2; x2=swap;
	  x1s=x1; y1s=y1;
	  while(x1 < x2+1 && y1< y2+1)
	  {
        if((double)(x1-x1s+1)==(double)(y1-y1s+1)*cotga)
	  	{
			write_a(x1t, y1);
			x1++; y1++; x1t--;
			continue;
		}
	    if((double)(x1-x1s+1)/cotga > (y1-y1s+1))
	  	{
	      write_a(x1t, y1);
	      y1++;
          continue;
		}
		if((double)(y1-y1s+1)*cotga > (x1-x1s+1))
		{
           write_a(x1t, y1);
           x1++;
           x1t--;
           continue;
        }
      }
	 }     
  	 draw_circle(); 
     LCDClear();
     LCDUpdate();  
     print (num);
     clear();
}
